/*
 * Copyright 2019 The Kubernetes Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as assert from 'assert';
import { waitTimeout } from './cli';
import * as Selectors from './selectors';
import { keys as Keys } from './keys';
import { expectArray, expectText, getValueFromMonaco, expectYAMLSubset } from './util';
import { blockAfter } from './repl-expect';
export const open = (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client
        .$(Selectors.SIDECAR(res.count, res.splitIndex))
        .then(_ => _.waitForDisplayed({ timeout: waitTimeout }));
    return res;
});
/** Same as open(), but in the block proceding the given block */
export function openInBlockAfter(res, delta = 1) {
    return open(blockAfter(res, delta));
}
export const notOpen = (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client
        .$(Selectors.SIDECAR(res.count, res.splitIndex))
        .then(_ => _.waitForDisplayed({ timeout: waitTimeout, reverse: true }));
    return res;
});
export const openWithFailure = (res) => __awaiter(void 0, void 0, void 0, function* () {
    return res.app.client
        .$(Selectors.SIDECAR_WITH_FAILURE(res.count, res.splitIndex))
        .then(_ => _.waitForDisplayed({ timeout: waitTimeout }))
        .then(() => res);
});
/** fully closed, not just minimized */
export const fullyClosed = (res) => __awaiter(void 0, void 0, void 0, function* () {
    return res.app.client
        .$(Selectors.SIDECAR_FULLY_CLOSED(res.count, res.splitIndex))
        .then(_ => _.waitForExist())
        .then(() => res);
});
/** either minimized or fully closed */
export const closed = (res) => __awaiter(void 0, void 0, void 0, function* () {
    return fullyClosed(res);
});
/** close the sidecar by ESCAPE key */
export const keyToClose = (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client.keys(Keys.ESCAPE);
    return closed(res);
});
/** Expect the given badge to exist in the sidecar header */
export const badge = (title, css, absent = false) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    let iter = 0;
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        const badges = css
            ? yield res.app.client.$(`${Selectors.SIDECAR_BADGES(res.count, res.splitIndex)} .${css}`)
            : yield res.app.client.$(Selectors.SIDECAR_BADGES(res.count, res.splitIndex));
        const badgeLabels = yield badges.getText();
        const idx = badgeLabels.indexOf(title);
        if (++iter > 5) {
            console.error(`still waiting for sidebar badge title='${title}' actualBadges=${badgeLabels} idx=${idx}`);
        }
        return !absent ? idx >= 0 : idx < 0;
    }), { timeout: waitTimeout });
    return res;
});
export const button = (button) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client
        .$(Selectors.SIDECAR_MODE_BUTTON(res.count, button.mode, res.splitIndex))
        .then(_ => _.waitForDisplayed({ timeout: waitTimeout }));
    return res;
});
export const mode = (expectedMode) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        yield res.app.client
            .$(`${Selectors.SIDECAR_MODE_BUTTON_SELECTED(res.count, expectedMode, res.splitIndex)}`)
            .then(_ => _.isDisplayed());
        return true;
    }), { timeout: waitTimeout });
    return res;
});
export const toolbarText = (expect) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield expectText(res.app, expect.text, expect.exact || false, expect.timeout)(Selectors.SIDECAR_TOOLBAR_TEXT(res.count, expect.type, res.splitIndex));
    return res;
});
export const toolbarAlert = (expect) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield expectText(res.app, expect.text, expect.exact)(Selectors.SIDECAR_ALERT(res.count, expect.type, res.splitIndex));
    return res;
});
const show = (expected, selector) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        return res.app.client
            .$(selector)
            .then(elt => elt.getText())
            .then(text => text === expected);
    }), { timeout: waitTimeout });
    return res;
});
export const name = (count, expectedName) => show(expectedName, Selectors.SIDECAR_TITLE(count));
export const namehash = (count, expectedNameHash) => show(expectedNameHash, Selectors.SIDECAR_ACTIVATION_TITLE(count));
export const namespace = (expectedNamespace) => (res) => __awaiter(void 0, void 0, void 0, function* () { return show(expectedNamespace, Selectors.SIDECAR_PACKAGE_NAME_TITLE(res.count))(res); });
export const kind = (expectedKind) => (res) => __awaiter(void 0, void 0, void 0, function* () { return show(expectedKind, Selectors.SIDECAR_KIND(res.count, res.splitIndex))(res); });
// expect sidecar tabs have correct `mode` and `label`
export const modes = (expected) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    return Promise.all(expected.map((_) => __awaiter(void 0, void 0, void 0, function* () {
        yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
            const actualModeSelector = `${Selectors.SIDECAR_MODE_BUTTON(res.count, _.mode, res.splitIndex)}`;
            const actualMode = yield res.app.client.$(actualModeSelector);
            yield actualMode.waitForExist();
            if (_.label && (yield actualMode.isDisplayed())) {
                const actualLabel = yield actualMode.getText();
                return actualLabel.toLowerCase() === _.label.toLowerCase();
            }
            else {
                return true;
            }
        }), { timeout: waitTimeout });
    }))).then(() => res);
});
// expect sidecar tab has the correct default tab
export const defaultMode = (expected) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        const actualModeSelector = `${Selectors.SIDECAR_MODE_BUTTON_SELECTED(res.count, expected.mode, res.splitIndex)}`;
        const actualMode = yield res.app.client.$(actualModeSelector);
        yield actualMode.waitForDisplayed();
        if (expected.label) {
            const actualLabel = yield actualMode.getText();
            return actualLabel.toLowerCase() === expected.label.toLowerCase();
        }
        else {
            return true;
        }
    }), { timeout: waitTimeout });
    return res;
});
export const textPlainContent = (content) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield expectText(res.app, content)(Selectors.SIDECAR_CUSTOM_CONTENT(res.count, res.splitIndex));
    return res;
});
export const textPlainContentFromMonaco = (expectedText, exact = true) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    let idx = 0;
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        const actualText = yield getValueFromMonaco(res);
        if (++idx > 5) {
            console.error(`still waiting for plain text from monaco actualText=${actualText} expectedText=${expectedText}`);
        }
        if (exact) {
            return actualText === expectedText;
        }
        else {
            return actualText.indexOf(expectedText) >= 0;
        }
    }), { timeout: waitTimeout });
    return res;
});
export function tableContent(res, nRows, nCells) {
    return __awaiter(this, void 0, void 0, function* () {
        const rows = yield res.app.client.$$(`${Selectors.SIDECAR_CUSTOM_CONTENT} tr`);
        assert.strictEqual(nRows, rows.length, 'nRows must match');
        const cells = yield res.app.client.$$(`${Selectors.SIDECAR_CUSTOM_CONTENT} td`);
        assert.strictEqual(nCells, cells.length, 'nCells must match');
    });
}
export const yaml = (content) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        const ok = yield getValueFromMonaco(res).then(expectYAMLSubset(content, false));
        return ok;
    }), { timeout: waitTimeout });
    return res;
});
export function popupTitle(res, expectedTitle) {
    return __awaiter(this, void 0, void 0, function* () {
        return res.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
            const actualTitle = yield res.app.client
                .$(Selectors.SIDECAR_POPUP_TITLE(res.count, res.splitIndex))
                .then(_ => _.getText());
            return actualTitle === expectedTitle;
        }), { timeout: waitTimeout });
    });
}
/** expect a form in the sidecar content */
export function form(form, idPrefix = 'kui-form') {
    return (res) => __awaiter(this, void 0, void 0, function* () {
        yield Promise.all(Object.keys(form).map(key => {
            return res.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
                const expectedValue = form[key];
                const actualValue = yield res.app.client
                    .$(`${Selectors.SIDECAR_TAB_CONTENT} #${idPrefix}-${key}`)
                    .then(_ => _.getValue());
                return actualValue === expectedValue;
            }), { timeout: waitTimeout });
        }));
        return res;
    });
}
export const showing = (expectedName, expectedActivationId, expectSubstringMatchOnName, expectedPackageName, expectType, waitThisLong, which, clickable) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        // check selected name in sidecar
        const sidecarSelector = `${Selectors.SIDECAR(res.count, res.splitIndex)}${!expectType ? '' : '.entity-is-' + expectType}`;
        yield res.app.client.$(sidecarSelector).then(_ => _.waitForDisplayed({ timeout: waitThisLong || waitTimeout }));
        // either 'leftnav' or 'topnav'
        if (!which) {
            which = (yield res.app.client.$(sidecarSelector).then(_ => _.getAttribute('data-view')));
        }
        const titleSelector = which === 'topnav'
            ? Selectors.SIDECAR_TITLE(res.count, res.splitIndex, clickable)
            : Selectors.SIDECAR_LEFTNAV_TITLE(res.count, res.splitIndex);
        return res.app.client
            .$(titleSelector)
            .then(_ => _.getText())
            .then(name => {
            const nameMatches = expectSubstringMatchOnName
                ? name.indexOf(expectedName) >= 0 || expectedName.indexOf(name) >= 0
                : name === expectedName;
            if (nameMatches) {
                if (expectedPackageName) {
                    return res.app.client
                        .$(Selectors.SIDECAR_PACKAGE_NAME_TITLE(res.count, res.splitIndex))
                        .then(_ => _.getText())
                        .then(name => expectSubstringMatchOnName
                        ? name.search(new RegExp(expectedPackageName, 'i')) >= 0
                        : name.toLowerCase() === expectedPackageName.toLowerCase());
                }
                else {
                    return true;
                }
            }
        });
    }), {
        timeout: waitThisLong || waitTimeout,
        timeoutMsg: `expect action name ${expectedName} in sidecar substringOk? ${expectSubstringMatchOnName}`
    });
    if (expectedActivationId) {
        yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
            try {
                const actualId = yield res.app.client
                    .$(Selectors.SIDECAR_ACTIVATION_TITLE(res.count, res.splitIndex))
                    .then(_ => _.getText());
                if (actualId === expectedActivationId) {
                    return true;
                }
                else {
                    console.error(`still waiting for nameHash; currently "${actualId}"; expecting "${expectedActivationId}"`);
                }
            }
            catch (err) {
                console.error(`error waiting for nameHash ${err.message}`, err);
            }
        }), { timeout: waitTimeout, timeoutMsg: `expect activation id ${expectedActivationId} in sidecar` });
    }
    return res;
});
export const showingNotClickable = (expectedName, expectedPackageName) => showing(expectedName, undefined, undefined, expectedPackageName, undefined, undefined, undefined, false);
export const showingTopNav = (expectedName) => showing(expectedName, undefined, undefined, undefined, undefined, undefined, 'topnav');
export const showingLeftNav = (expectedName) => showing(expectedName, undefined, undefined, undefined, undefined, undefined, 'leftnav');
export function breadcrumbs(breadcrumbs) {
    return (res) => __awaiter(this, void 0, void 0, function* () {
        const elts = yield res.app.client.$$(Selectors.SIDECAR_BREADCRUMBS(res.count, res.splitIndex));
        yield Promise.all(elts.map((elt) => __awaiter(this, void 0, void 0, function* () {
            yield elt.waitForDisplayed();
            return elt.getText();
        }))).then(expectArray(breadcrumbs));
        return res;
    });
}
/** Expect DescriptionList content in the current tab */
export const descriptionList = (content) => (res) => __awaiter(void 0, void 0, void 0, function* () {
    const container = res.splitIndex !== undefined
        ? `${Selectors.PROMPT_BLOCK_N_FOR_SPLIT(res.count, res.splitIndex)} .kui--tab-content:not([hidden])`
        : `${Selectors.PROMPT_BLOCK_N(res.count)} .kui--tab-content:not([hidden])`;
    for (const term in content) {
        let idx = 0;
        yield res.app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
            const expectedDescription = content[term].toString(); // the actualDescription will come as a string
            const actualDescription = yield res.app.client
                .$(`${container} ${Selectors.DLIST_DESCRIPTION_FOR(term)}`)
                .then(_ => _.getText());
            if (actualDescription !== expectedDescription) {
                if (++idx > 5) {
                    console.error(`Still waiting for expectedDescription=${expectedDescription} actualDescription=${actualDescription} `);
                }
                return false;
            }
            else {
                return true;
            }
        }), { timeout: waitTimeout });
    }
    return res;
});
//# sourceMappingURL=sidecar-expect.js.map