/*
 * Copyright 2019 The Kubernetes Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as assert from 'assert';
import { v4 as uuid } from 'uuid';
import { load } from 'js-yaml';
import * as Selectors from './selectors';
import * as CLI from './cli';
import * as Common from './common';
import * as ReplExpect from './repl-expect';
import * as SidecarExpect from './sidecar-expect';
import { keys } from './keys';
/**
 * subset means that it is ok for struct1 to be a subset of struct2
 * so: every key in struct1 must be in struct2, but not vice versa
 *
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const sameStruct = (struct1, struct2, subset = false) => {
    if (struct1 === struct2) {
        return true;
    }
    else if (typeof struct1 !== typeof struct2) {
        return false;
    }
    for (const key in struct1) {
        if (!(key in struct2)) {
            console.log(`!(${key} in struct2)`);
            return false;
        }
        else if (struct1[key].constructor === RegExp) {
            // then we have a validator regular expression
            if (!struct1[key].test(struct2[key])) {
                return false;
            }
        }
        else if (typeof struct1[key] === 'function') {
            // then we have a validator function
            if (!struct1[key](struct2[key])) {
                return false;
            }
        }
        else if (typeof struct1[key] !== typeof struct2[key]) {
            console.log(`typeof struct1[${key}] !== typeof struct2[${key}] ${typeof struct1[key]} ${typeof struct2[key]}`);
            console.log(struct1);
            console.log(struct2);
            return false;
        }
        else if (typeof struct1[key] === 'object') {
            if (!sameStruct(struct1[key], struct2[key], subset)) {
                return false;
            }
        }
        else if (struct1[key] !== struct2[key]) {
            console.log(`struct1[${key}] !== struct2[${key}] ${struct1[key]} ${struct2[key]}`);
            return false;
        }
    }
    // if struct1 if expected to be a subset of struct2, then we're done
    if (subset)
        return true;
    for (const key in struct2) {
        if (!(key in struct1)) {
            console.log(`!(${key} in struct1)`);
            return false;
        }
        else if (typeof struct1[key] === 'function') {
            // then we have a validator function
            if (!struct1[key](struct2[key])) {
                return false;
            }
        }
        else if (typeof struct1[key] !== typeof struct2[key]) {
            console.log(`typeof struct1[${key}] !== typeof struct2[${key}] ${typeof struct1[key]} ${typeof struct2[key]}`);
            console.log(struct1);
            console.log(struct2);
            return false;
        }
        else if (typeof struct2[key] === 'object') {
            if (!sameStruct(struct1[key], struct2[key], subset)) {
                return false;
            }
        }
        else if (struct1[key] !== struct2[key]) {
            console.log(`struct1[${key}] !== struct2[${key}] ${struct1[key]} ${struct2[key]}`);
            return false;
        }
    }
    return true;
};
export const expectSubset = (struct1, failFast = true) => (str) => {
    try {
        const ok = sameStruct(struct1, JSON.parse(str), true);
        if (failFast) {
            assert.ok(ok);
        }
        return true;
    }
    catch (err) {
        console.error('Error comparing subset for actual value=' + str);
        throw err;
    }
};
/** is the given struct2 the same as the given struct2 (given as a string) */
export const expectStruct = (struct1, noParse = false, failFast = true) => (str) => {
    try {
        const ok = sameStruct(struct1, noParse ? str : JSON.parse(str));
        if (failFast) {
            assert.ok(ok);
        }
        return ok;
    }
    catch (err) {
        console.error('Error comparing structs for actual value=' + str);
        throw err;
    }
};
export const expectYAML = (struct1, subset = false, failFast = true) => (str) => {
    try {
        const struct2 = load(str);
        const ok = sameStruct(struct1, typeof struct2 !== 'string' || typeof struct2 !== 'boolean' || typeof struct2 !== 'number'
            ? struct2
            : JSON.parse(typeof struct2 === 'number' ? `${struct2}` : struct2), subset);
        if (failFast) {
            assert.ok(ok);
        }
        return ok;
    }
    catch (err) {
        if (failFast) {
            return false;
        }
        else {
            console.error('Error comparing subset for actual value=' + str);
            throw err;
        }
    }
};
export const expectYAMLSubset = (struct1, failFast = true) => expectYAML(struct1, true, failFast);
/** is the given actual array the same as the given expected array? */
export const expectArray = (expected, failFast = true, subset = false) => (actual) => {
    if (!Array.isArray(actual)) {
        // webdriver.io's getText will return a singleton if there is only one match
        actual = [actual];
    }
    const matchFn = function (u, i) {
        return u === expected[i];
    };
    const ok = !subset ? actual.length === expected.length && actual.every(matchFn) : actual.some(matchFn);
    if (!ok) {
        console.error(`array mismatch; expected=${expected} actual=${actual}`);
    }
    if (failFast) {
        assert.ok(ok);
    }
    else {
        return ok;
    }
};
/** Expand the fold on the given line of a monaco editor */
export function clickToExpandMonacoFold(res, lineIdx) {
    return __awaiter(this, void 0, void 0, function* () {
        const container = res.splitIndex !== undefined
            ? `${Selectors.PROMPT_BLOCK_N_FOR_SPLIT(res.count, res.splitIndex)} .kui--tab-content:not([hidden])`
            : `${Selectors.PROMPT_BLOCK_N(res.count)} .kui--tab-content:not([hidden])`;
        console.log('MF1');
        const selector = `${container} .monaco-editor-wrapper`;
        const wrapper = yield res.app.client.$(selector);
        yield wrapper.waitForExist({ timeout: CLI.waitTimeout });
        console.log('MF2');
        const marginOverlaysSelector = `.margin-view-overlays > div:nth-child(${lineIdx + 1})`; // css is 1-indexed
        const foldingSelector = `${marginOverlaysSelector} .codicon-folding-collapsed`;
        const foldingClickable = yield wrapper.$(foldingSelector);
        console.log('MF3');
        yield foldingClickable.waitForExist({ timeout: CLI.waitTimeout });
        console.log('MF4');
        yield foldingClickable.click();
        console.log('MF5');
        const unfoldingSelector = `${marginOverlaysSelector} .codicon-folding-expanded`;
        const unfoldingClickable = yield wrapper.$(unfoldingSelector);
        yield unfoldingClickable.waitForExist({ timeout: CLI.waitTimeout });
        console.log('MF6');
    });
}
/** get the monaco editor text */
export const getValueFromMonaco = (res, container) => __awaiter(void 0, void 0, void 0, function* () {
    if (!container) {
        container =
            res.splitIndex !== undefined
                ? `${Selectors.PROMPT_BLOCK_N_FOR_SPLIT(res.count, res.splitIndex)} .kui--tab-content:not([hidden])`
                : `${Selectors.PROMPT_BLOCK_N(res.count)} .kui--tab-content:not([hidden])`;
    }
    const selector = `${container} .monaco-editor-wrapper`;
    try {
        yield res.app.client.$(selector).then(_ => _.waitForExist({ timeout: CLI.waitTimeout }));
    }
    catch (err) {
        console.error('cannot find editor', err);
        yield res.app.client
            .$(Selectors.SIDECAR(res.count, res.splitIndex))
            .then(_ => _.getHTML())
            .then(html => {
            console.log('here is the content of the sidecar:');
            console.log(html);
        });
        throw err;
    }
    return res.app.client.execute(selector => {
        try {
            return document.querySelector(selector).getValueForTests();
        }
        catch (err) {
            console.error('error in getValueFromMonaco1', err);
            // intentionally returning undefined
        }
    }, selector);
});
export const waitForXtermInput = (app, N) => {
    const selector = `${Selectors.PROMPT_BLOCK_N(N)} .xterm-helper-textarea`;
    return app.client.$(selector).then(_ => _.waitForExist());
};
export const expectText = (app, expectedText, exact = true, timeout = CLI.waitTimeout) => (selector) => __awaiter(void 0, void 0, void 0, function* () {
    let idx = 0;
    yield app.client.waitUntil(() => __awaiter(void 0, void 0, void 0, function* () {
        const actualText = yield app.client.$(selector).then(_ => _.getText());
        if (++idx > 5) {
            console.error(`still waiting for text; actualText=${actualText} expectedText=${expectedText} selector=${selector}`);
        }
        if (exact) {
            return actualText === expectedText;
        }
        else {
            if (actualText.indexOf(expectedText) < 0) {
                console.error(`Expected string not found: expected=${expectedText} actual=${actualText}`);
            }
            return actualText.indexOf(expectedText) >= 0;
        }
    }), { timeout });
    return app;
});
/** @return the current number of tabs */
export function tabCount(app) {
    return __awaiter(this, void 0, void 0, function* () {
        const topTabs = yield app.client.$$(Selectors.TOP_TAB);
        return topTabs.length;
    });
}
/** Close all except the first tab */
export function closeAllExceptFirstTab(expectedInitialNTabs = 2) {
    it('should close all but first tab', () => __awaiter(this, void 0, void 0, function* () {
        try {
            yield this.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
                const currentCount = yield tabCount(this.app);
                return currentCount === expectedInitialNTabs;
            }));
            let nTabs = yield tabCount(this.app);
            while (nTabs > 1) {
                const N = nTabs--;
                yield this.app.client.$(Selectors.TOP_TAB_CLOSE_N(N)).then(_ => _.click());
                yield this.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
                    const currentCount = yield tabCount(this.app);
                    return currentCount === N - 1;
                }));
            }
        }
        catch (err) {
            yield Common.oops(this, true)(err);
        }
    }));
}
export function uniqueFileForSnapshot() {
    return `/tmp/${uuid()}.kui`;
}
/** Click the specified button action button */
export function clickBlockActionButton(res, buttonSelector) {
    return __awaiter(this, void 0, void 0, function* () {
        const N = res.count;
        const prompt = yield res.app.client.$(Selectors.PROMPT_N(N));
        yield prompt.scrollIntoView();
        yield prompt.moveTo();
        const removeButton = yield res.app.client.$(buttonSelector);
        yield removeButton.scrollIntoView();
        yield removeButton.waitForDisplayed({ timeout: CLI.waitTimeout });
        yield removeButton.moveTo();
        yield removeButton.click();
    });
}
/** Click the close button on a block, and expect it to be gone */
export function removeBlock(res) {
    return __awaiter(this, void 0, void 0, function* () {
        return clickBlockActionButton(res, Selectors.BLOCK_REMOVE_BUTTON(res.count));
    });
}
/** Click the block copy button */
export function copyBlockLink(res) {
    return __awaiter(this, void 0, void 0, function* () {
        return clickBlockActionButton(res, Selectors.BLOCK_LINK_BUTTON(res.count));
    });
}
/** Click the block run button */
export function rerunCommand(res) {
    return __awaiter(this, void 0, void 0, function* () {
        return clickBlockActionButton(res, Selectors.COMMAND_RERUN_BUTTON(res.count));
    });
}
/** Switch sidecar tab */
export function switchToTab(mode) {
    return (res) => __awaiter(this, void 0, void 0, function* () {
        const tab = yield res.app.client.$(Selectors.SIDECAR_MODE_BUTTON(res.count, mode, res.splitIndex));
        yield tab.waitForDisplayed();
        yield tab.click();
        yield res.app.client
            .$(Selectors.SIDECAR_MODE_BUTTON_SELECTED(res.count, mode, res.splitIndex))
            .then(_ => _.waitForDisplayed());
        return res;
    });
}
/** Output of the given block */
export function outputOf(res) {
    return res.app.client.$(Selectors.OUTPUT_N(res.count, res.splitIndex)).then(_ => _.getText());
}
/**
 * Execute `command` and expect a table with `name`
 *
 */
export function doList(ctx, command, name) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const tableRes = yield CLI.command(command, ctx.app);
            return ReplExpect.okWithCustom({ selector: Selectors.BY_NAME(name) })(tableRes);
        }
        catch (err) {
            yield Common.oops(ctx, true)(err);
        }
    });
}
export function openSidecarByClick(ctx, selector, name, mode, activationId, splitIndex = 2, // after we click, we expect two splits
inNotebook = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const app = ctx.app;
        // Note! if we already have 2 splits, we need to grab the count before we click! see https://github.com/IBM/kui/issues/6636
        const currentSplitCount = (yield app.client.$$(Selectors.SPLITS)).length;
        const currentLastBlockIdx = currentSplitCount === splitIndex ? (yield CLI.lastBlock(app, splitIndex, 1, inNotebook)).count : -1;
        // now click on the table row
        const clickOn = yield app.client.$(selector);
        yield clickOn.waitForExist({ timeout: CLI.waitTimeout });
        yield clickOn.scrollIntoView(false);
        yield clickOn.click();
        // expect 2 splits in total
        yield ReplExpect.splitCount(splitIndex)(app);
        // expect sidecar shown in the last blck of the second split
        yield app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
            const sidecarRes = yield CLI.lastBlock(app, splitIndex, 1, inNotebook);
            return sidecarRes.count > currentLastBlockIdx;
        }));
        const sidecarRes = yield CLI.lastBlock(app, splitIndex, 1, inNotebook);
        yield SidecarExpect.open(sidecarRes).then(SidecarExpect.showing(name, activationId));
        if (mode) {
            yield SidecarExpect.mode(mode)(sidecarRes);
        }
        return sidecarRes;
    });
}
/**
 * Execute `command` and expect a table with `name`,
 * and then open sidecar by clicking the `name`
 *
 */
export function listAndOpenSidecarNoWait(ctx, command, name, mode) {
    return __awaiter(this, void 0, void 0, function* () {
        const selector = `${yield doList(ctx, command, name)} [data-value="${name}"].clickable`;
        return openSidecarByClick(ctx, selector, name, mode);
    });
}
/**
 * Click sidecar mode button by `mode`
 *
 */
export function clickSidecarModeButton(ctx, res, mode) {
    return __awaiter(this, void 0, void 0, function* () {
        return ctx.app.client.$(Selectors.SIDECAR_MODE_BUTTON(res.count, mode, res.splitIndex)).then((_) => __awaiter(this, void 0, void 0, function* () {
            yield _.waitForDisplayed();
            // work around tooltips getting in the way of the click
            yield ctx.app.client.$(Selectors.STATUS_STRIPE).then(_ => _.moveTo());
            yield new Promise(resolve => setTimeout(resolve, 1000));
            yield _.click();
        }));
    });
}
/**
 * Click a sidecar button with `selector`
 *
 */
export function clickSidecarButtonCustomized(ctx, res, selector) {
    return __awaiter(this, void 0, void 0, function* () {
        return ctx.app.client.$(`${Selectors.SIDECAR(res.count, res.splitIndex)} ${selector}`).then((_) => __awaiter(this, void 0, void 0, function* () {
            yield _.waitForDisplayed();
            yield _.click();
        }));
    });
}
export function doCancel(cmd = '') {
    return this.app.client
        .$(Selectors.CURRENT_PROMPT_BLOCK)
        .then((_) => __awaiter(this, void 0, void 0, function* () {
        yield _.waitForExist();
        return _.getAttribute('data-input-count');
    }))
        .then(count => parseInt(count, 10))
        .then(count => this.app.client
        .keys(cmd)
        .then(() => this.app.client.keys(keys.ctrlC))
        .then(() => ({ app: this.app, count }))
        .then(ReplExpect.blank)
        .then(() => this.app.client.$(Selectors.PROMPT_N(count))) // make sure the cancelled command text is still there, in the previous block
        .then(_ => _.getText())
        .then(input => assert.strictEqual(input, cmd)))
        .catch(Common.oops(this, true));
}
export function switchToTopLevelTabViaClick(ctx, N) {
    return ctx.app.client
        .$(Selectors.TOP_TAB_N_CLICKABLE(N))
        .then(_ => _.click())
        .then(() => ctx.app.client.$(Selectors.TAB_SELECTED_N(N)))
        .then(_ => _.waitForDisplayed())
        .catch(Common.oops(ctx, true));
}
export function doClear(residualBlockCount = 1, splitIndex = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        yield this.app.client.keys([keys.CONTROL, 'l', 'NULL']); // use control-l to clear
        yield ReplExpect.consoleToBeClear(this.app, residualBlockCount, splitIndex);
    });
}
export function clickNewTabButton(ctx, expectedNewTabIndex) {
    return ctx.app.client
        .$(Selectors.tabButtonSelector)
        .then(_ => _.click())
        .then(() => ctx.app.client.$(Selectors.TAB_SELECTED_N(expectedNewTabIndex)))
        .then(_ => _.waitForDisplayed())
        .then(() => CLI.waitForRepl(ctx.app)); // should have an active repl
}
export function getTabTitle(ctx, tabTitleSelector) {
    return __awaiter(this, void 0, void 0, function* () {
        const tabTitle = yield ctx.app.client.$(tabTitleSelector);
        yield tabTitle.waitForDisplayed({ timeout: CLI.waitTimeout });
        const actualTabTitle = yield tabTitle.getValue();
        return actualTabTitle;
    });
}
export function expectCurrentTabTitle(ctx, expectedTabTitle, tabTitleSelector = Selectors.CURRENT_TAB_TITLE) {
    return ctx.app.client.waitUntil(() => __awaiter(this, void 0, void 0, function* () {
        const actualTabTitle = yield getTabTitle(ctx, tabTitleSelector);
        return actualTabTitle === expectedTabTitle;
    }), { timeout: CLI.waitTimeout });
}
//# sourceMappingURL=util.js.map